require 'digest/sha1'
require 'RMagick'
class User < ActiveRecord::Base
	belongs_to :role
	has_many :comments, :dependent => :destroy
	has_many :ads, :dependent => :destroy
	has_many :reviews, :dependent => :destroy
	
	attr_accessor :uploaded_photo, :password
	N_("User|Password")
  N_("User|Password confirmation")
  N_("User|Role")

	validates_presence_of :name, :surname, :birth_date, :username, :email
	validates_presence_of :password, :if => :password_required?
	validates_presence_of :password_confirmation, :if => :password_required?
	
	validates_length_of :username, :within => 3..40, :if => :has_username?
	validates_length_of :email, :within => 3..100, :if => :has_email?
	validates_length_of :password, :within => 4..40, :if => :password_required? && :has_password?
	validates_confirmation_of :password, :if => :password_required?
	validates_format_of :email, :if => :has_email?, :with => /^([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})$/i
	validates_uniqueness_of :username, :case_sensitive => false, :if => :has_username?
	validates_uniqueness_of :email, :case_sensitive => false, :if => :has_email?
	validates_uniqueness_of :mobile_phone
	validates_length_of :mobile_phone, :is => 9
	validates_numericality_of :mobile_phone
	
	before_save :encrypt_password
	before_save :save_photo

	# messages
	has_many :messages_as_sender, :foreign_key => 'sender_id', :class_name => 'Message'
	has_many :messages_as_receiver, :foreign_key => 'receiver_id', :class_name => 'Message'
	has_many :users_who_messaged_me, :through => :messages_as_receiver, :source => :sender
	has_many :users_whom_i_messaged, :through => :messages_as_sender, :source => :receiver
  
  def requested(ad)
    requests = AdRequest.find_all_by_ad_id_and_user_id(ad.id, self.id)
    return true if requests.length>0
    return false
  end
  
  def new_messages
    return self.messages_as_receiver_count
  end
  
	def photo
		photo_dir = "#{RAILS_ROOT}/public/uploads/user_photos"
		nophoto = "no_photo.gif"
		File.exists?("#{photo_dir}/#{self.id}.jpg") ? "/uploads/user_photos/#{self.id}.jpg" : "#{nophoto}"
	end

  # authenticates a user by their login name and unencrypted password.  Returns the user or nil.
  def self.authenticate(email, password)
    u = find_by_email(email) # need to get the salt
    u && u.authenticated?(password) ? u : nil
  end

  # encrypts some data with the salt.
  def self.encrypt(password, salt)
    Digest::SHA1.hexdigest("--#{salt}--#{password}--")
  end

  # encrypts the password with the user salt
  def encrypt(password)
    self.class.encrypt(password, salt)
  end

  def authenticated?(password)
    crypted_password == encrypt(password)
  end

  def remember_token?
    remember_token_expires_at && Time.now.utc < remember_token_expires_at 
  end

  # these create and unset the fields required for remembering users between browser closes
  def remember_me
    self.remember_token_expires_at = 2.weeks.from_now.utc
    self.remember_token            = encrypt("#{email}--#{remember_token_expires_at}")
    save(false)
  end

  def forget_me
    self.remember_token_expires_at = nil
    self.remember_token            = nil
    save(false)
  end

  # added to support roles
  def has_role?(rolename)
    self.role.name.eql?(rolename) ? true : false 
  end

	def can_edit?(ad)
		self.eql?ad.user || self.has_role?('Admin')
	end

	# save user photo
	def save_photo
		if uploaded_photo
			photo_name = "_#{uploaded_photo.original_filename}"
			photo_dir = "#{RAILS_ROOT}/public/uploads/user_photos"
			FileUtils.makedirs("#{photo_dir}")
			File.open("#{photo_dir}/#{photo_name}", "w") { |f| f.write(uploaded_photo.read) }
			
			# resize
			@temp = Magick::Image.read("#{photo_dir}/#{photo_name}").first
			out = @temp.change_geometry("100x100"){ |cols, rows, img| img.resize!(cols, rows)}
			out.write("#{photo_dir}/#{self.id}.jpg")

			# delete original
			FileUtils.rm_r("#{photo_dir}/#{photo_name}")
		end
	end

	# zodiac sign
	def sign
		signs = [_("Aquarius"), _("Pisces"), _("Aries"), _("Taurus"), _("Gemini"), _("Cancer"),
						_("Leo"), _("Virgo"), _("Libra"), _("Scorpio"), _("Sagittarius"), _("Capricorn")]

		dates = [["01-21", "02-19"], ["02-20", "03-20"], ["03-21", "04-20"], ["04-21", "05-20"],
						["05-21", "06-21"], ["06-22", "07-21"], ["07-22", "08-23"], ["08-24", "09-22"],
						["09-23", "10-22"], ["10-23", "11-21"], ["11-22", "12-21"]]
		
		bday = birth_date.to_date.to_default_s.split("-")
		bday.shift
		bday = bday.join("-")
		
		dates.length.times do |i|
			if bday.between?(dates[i].first, dates[i].last)
				return signs[i]
			end
		end
		
		return signs.last
	end
  
  def full_name
    return self.name+" "+self.surname
  end
  
	def gender?
		self.gender ? _("Male") : _("Female")
	end

	def smoker?
		self.smoker ? _("yes").capitalize : _("no").capitalize
	end
	
	def pets?
		self.pets ? _("yes").capitalize : _("no").capitalize
	end

	def age
		age = Date.today.year - self.birth_date.to_date.year
		age -= 1 if self.birth_date.to_date > Date.today.years_ago(age)
		age
	end

	def forgot_password
		@forgotten_password = true
		self.make_password_reset_code
	end

	def reset_password
		# First update the password_reset_code before setting them
		# reset_password flag to avoid duplicate email notifications.
		update_attributes(:password_reset_code => nil)
		@reset_password = true
	end

	def recently_reset_password?
		@reset_password
	end
	
	def recently_forgot_password?
		@forgotten_password
	end

  protected
		def has_username?
			!username.blank?
		end
		
		def has_email?
			!email.blank?
		end
		
		def has_password?
			!password.blank?
		end
    
		def make_password_reset_code
			self.password_reset_code = Digest::SHA1.hexdigest(Time.now.to_s.split(//).sort_by {rand}.join)
		end

		# before filter 
    def encrypt_password
      return if password.blank?
      self.salt = Digest::SHA1.hexdigest("--#{Time.now.to_s}--#{username}--") if new_record?
      self.crypted_password = encrypt(password)
    end
    
    def password_required?
      crypted_password.blank? || !password.blank?
    end
end
