require File.dirname(__FILE__) + '/../spec_helper'

describe User do
  it "depends on RMagick" do
    lambda {
      begin
        require "RMagick"
      rescue LoadError, MissingSourceFile => e
        raise e, "Make sure RMagick is available"
      end
    }.should_not raise_error
  end  
end

context "A new User" do
  specify "should be valid with a full set of valid attributes" do
    lambda{ 
        user = create_user
        user.should_not be_new_record 
    }.should change(User,:count).by(1)
  end
  
  specify "should be invalid without a name" do
    lambda{
      u = create_user(:name => nil)
      u.should have_at_least(1).errors_on(:name)
    }.should_not change(User, :count)
  end
  
  specify "should be invalid without a surname" do
    lambda{
      u = create_user(:surname => nil)
      u.should have_at_least(1).errors_on(:surname)
    }.should_not change(User, :count)
  end
  
  specify "should calculate correct age" do
    16.upto(90) do |age|
      u = create_user(:birth_date => Time.now-age.years)
      u.age.should be_eql(38)
    end
  end
  
  specify "should be invalid without an username" do
    lambda{ 
      u = create_user(:username => nil)
      u.should have_at_least(1).errors_on(:username) 
    }.should_not change(User,:count)
  end
  
  specify "should be invalid without a password" do
    lambda{ 
      u = create_user(:password => nil)
      u.should have_at_least(1).errors_on(:password)
    }.should_not change(User,:count)
  end

  specify "should be invalid without a password confirmation" do
    lambda{ 
      u = create_user(:password_confirmation => nil)
      u.should have_at_least(1).errors_on(:password_confirmation)
    }.should_not change(User,:count)
  end

  specify "should be invalid without an email" do
    lambda{ 
      u = create_user(:email => nil)
      u.should have_at_least(1).errors_on(:email)
    }.should_not change(User,:count)
  end
  
  specify "should be invalid with an invalid email" do
    lambda{ 
      u = create_user(:email => "invalid@com")
      u.should have_at_least(1).errors_on(:email)
      u = create_user(:email => "@invalid.com")
      u.should have_at_least(1).errors_on(:email)
      u = create_user(:email => "*@*.com")
      u.should have_at_least(1).errors_on(:email)
    }.should_not change(User,:count)
  end
  
  specify "should be invalid without a birth date" do
    lambda{
      u = create_user(:birth_date => nil)
      u.should have_at_least(1).errors_on(:birth_date)
    }.should_not change(User, :count)
  end
  
  specify "should have a default image on filesystem" do
    default_pic = "no_photo.gif"
    File.exists?("#{RAILS_ROOT}/public/images/#{default_pic}").should be_true
  end
  
  specify "should have the default image when created" do
    lambda {
      u = create_user
      default_pic = "no_photo.gif"
      u.photo.should be_eql(default_pic)
    }.should change(User,:count)
  end
  
  protected
  def create_user(options = {})
		User.create({
			:username => 'quire',
			:name => 'quire',
			:surname => 'qi',
			:birth_date => Time.now,
			:email => 'quire@example.com',
			:password => 'quire',
			:password_confirmation => 'quire'
		}.merge(options))
	end
end

context "User with fixtures loaded" do
  fixtures :users

  specify "should be able to reset password" do
    users(:quentin).update_attributes(:password => 'new password', :password_confirmation => 'new password')
    users(:quentin).should == User.authenticate(users(:quentin).email,'new password')
  end

  specify "should not rehash password on attribute updates" do
    users(:quentin).update_attributes(:email => 'quentin2@hotmail.com')
    users(:quentin).should == User.authenticate('quentin2@hotmail.com','test')
  end

  specify "should authenticate valid user" do
    users(:quentin).should == User.authenticate(users(:quentin).email,'test')
  end
end